# -*- coding: utf-8 -*-
"""
Created on Tue Feb 27 08:38:02 2024

@author: Braan
"""
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import interp1d
from scipy.optimize import fsolve

plt.rcParams.update({
    "text.usetex": True,
    'font.family': 'serif',  # Set font family
    'font.size': 15,          # Set font size
    'axes.labelsize': 17,     # Set label font size
    'axes.titlesize': 20,     # Set title font size
    'xtick.labelsize': 20,    # Set x-axis tick label font size
    'ytick.labelsize': 20,     # Set y-axis tick label font size
    'legend.borderaxespad':0.4 # the border between the axes and legend edge
})

# Load data from CSV, skip the first row (header), replace "," with "."
file_path = 'MeOH_fractions.csv'

feed_stage = 34 #feed inlet stage (tray)
x_zf = 0.5 #liquid light component mole fraction in feed


df = pd.read_csv(file_path, sep=';', header=None, names=['Liquid', 'Vapour'], skiprows=1)
df['Liquid'] = df['Liquid'].str.replace(',', '.').astype(float)
df['Vapour'] = df['Vapour'].str.replace(',', '.').astype(float)


# Extract columns
x_measured = df['Liquid'].tolist()
y_measured = df['Vapour'].tolist()

xDL = x_measured[1]    # Distillate composition
xBL = x_measured[-2]    # Bottoms composition

op_line_x = x_measured[:-1]
op_line_y = y_measured[1:]

rect_line_x = op_line_x[0:feed_stage]
rect_line_y = op_line_y[0:feed_stage]
rect_DY = (op_line_y[feed_stage-1:feed_stage][0]-op_line_y[feed_stage-2:feed_stage-1][0])
rect_DX = (op_line_x[feed_stage-1:feed_stage][0]-op_line_x[feed_stage-2:feed_stage-1][0])

try:
    rect_line_grad = rect_DY/rect_DX
    RR = rect_line_grad/(1-rect_line_grad)
except ZeroDivisionError:
    RR = "infinite"

strip_line_x = op_line_x[feed_stage:]
strip_line_y = op_line_y[feed_stage:]
strip_DY = (op_line_y[feed_stage:feed_stage+1][0]-op_line_y[feed_stage+1:feed_stage+2][0])
strip_DX = (op_line_x[feed_stage:feed_stage+1][0]-op_line_x[feed_stage+1:feed_stage+2][0])
strip_line_grad = strip_DY/strip_DX


#Interpolating the curves
equilibrium_curve = interp1d(x_measured, y_measured, kind='linear', fill_value="extrapolate")
rect_curve = interp1d(rect_line_x, rect_line_y, kind='linear', fill_value="extrapolate")
strip_curve = interp1d(strip_line_x, strip_line_y, kind='linear', fill_value="extrapolate")

intersection_function = lambda x: rect_curve(x) - strip_curve(x)
x0 = 0.5

x_q1 = fsolve(intersection_function, x0)[0]
y_q1 = rect_curve(x_q1)

rect_line_x.insert(-1,x_q1)
rect_line_y.insert(-1,y_q1)
strip_line_x.insert(0,x_q1)
strip_line_y.insert(0,y_q1)

x_q = [x_zf,x_q1]
y_q = [x_zf,y_q1]
try:
    q_grad = (y_q[0]-y_q[1])/(x_q[0]-x_q[1])
    q = q_grad/(q_grad-1)
except ZeroDivisionError:
    q = 1
# Plotting
x_range = np.linspace(min(x_measured), max(x_measured), 100)
y_eq_curve_interpolated = equilibrium_curve(x_range)

plt.figure(figsize=(16, 8),dpi=450)
plt.grid(which='major', linestyle='-', linewidth='0.8', color='black', alpha=0.3)
plt.xticks(np.arange(0, 1.0, 0.05))
plt.yticks(np.arange(0, 1.0, 0.05))

plt.grid(which='minor', linestyle=':', linewidth='0.8', color='black', alpha=0.3)
plt.minorticks_on()
plt.xticks(np.arange(0, 1.01, 0.05))
plt.yticks(np.arange(0, 1.01, 0.05))
plt.xlim((0,1))
plt.ylim((0,1))

plt.step(x_measured, y_measured, where='pre',linewidth='0.9',color = 'black')
plt.plot(x_measured, y_measured, '.', label='Data points')
plt.plot(x_range, y_eq_curve_interpolated, label='Interpolated Equilibrium Curve (linear)',color = 'black',alpha=0.5)
plt.plot(rect_line_x,rect_line_y,'-',label = f"Rectifying line ",color = 'red',alpha=0.6) #(L/F = {RR:.2f})
plt.plot(strip_line_x,strip_line_y,'-',label = "Stripping line",color = 'blue',alpha=0.6)
plt.plot(x_q,y_q,'g-',label=f'q-line, (q = {q:.2f})')
plt.plot([0, 1], [0, 1],'--',c='black')

plt.axvline(x=xDL, color='r', linestyle='--', label=r'$x_{DL}$'+ f' = {xDL:.4f}',alpha=0)
plt.axvline(x=xBL, color='b', linestyle='--', label=r'$x_{BL}$'+ f' = {xBL:.4f}',alpha=0)

plt.xlabel('Liquid Fraction of MeOH')
plt.ylabel('Vapour Fraction of MeOH')
plt.title(f'McCabe-Thiele Diagram for MeOH-Water   \nFeed stage:'f' {feed_stage}')

plt.legend(loc='best', fontsize='large', fancybox=True, framealpha=0.65, borderpad=0.4,  handlelength=3, handleheight=1.6, ncol=1, labelspacing=0.0, columnspacing=0, handletextpad=1, borderaxespad=1, edgecolor='black', facecolor='white', markerscale=2)
plt.show()
