import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter
import numpy as np
from clc_extractor import export_data, directory_path, time_unit
from grad import nth_numerical_gradient
import os

mpl.rcParams.update({
    "text.usetex": True,
    'font.family': 'serif',  # Set font family
    'font.size': 24,          # Set font size [HANDLES]
    'axes.labelsize': 30,     # Set label font size [X,Y LABEL]
    'axes.titlesize': 35,     # Set title font size [TITLES]
    'xtick.labelsize': 26,    # Set x-axis tick label font size
    'ytick.labelsize': 26,     # Set y-axis tick label font size
    'legend.borderaxespad':0.2, # the border between the axes and legend edge
    'lines.linewidth':4
})


#main results
xs_dict = ({
    'AD0': 'AD0.3r1',
    'AD1': 'AD1.9r3',
    'AD2': 'AD2.9r3',
    'AD3': 'AD3.6r2',
    'AD4': 'AD4.6r3',
    'AD5': 'AD5.1r3',
    'AD6': 'AD7.1r3'
    })


#PLOT OPTIONS
sim_name = 'AD5.1r4'  # Aspen Dynamics simulation scenario (case sensitive)
purity = 0.999 #target purity
time_0 = 0 #depends on the input time unit from clc_extractor.py
time_end = -1 #same as above. set to -1 if no data is to be cut
purity_scale = 'log10' #log10, linear, both (both does not work YET)
include_LC = False #Include level control plot?
only_xs = False #only include purity plots from all experiments given in xs_dict. Disables "sim_name"
compare_to = False # compare to key in xs_dict (eg. "AD0"). Disable by "False"
tick_format = '' # scientific, normal, None


"""
export_data() dictionary
'x':t (data)
'y':variables (data)
'y_names': variable_names (string)
"""

def format_ticks(value, pos): #format numbers on axis
    if 'sci' in tick_format:
        return "%.2e" % value
    elif 'nor' in tick_format:
        return "%.2f" % value
    else:
        return None

def check_directory(directory_path): #Finds the directories of the desired files that contain {sim_name} in its' name
    if os.path.exists(directory_path):
        files = os.listdir(directory_path)
        # print(f"Files in the directory: {files}")
        return files
    else:
        print(f"Directory {directory_path} does not exist.")
        return files
    
files = check_directory(directory_path)

def plot(sim_name):
    # Establishes what files are to be rendered
    indices_dict = {}
    if only_xs:
        xs_dict_mod = {key + "": value for key, value in xs_dict.items()}
        for key, value in xs_dict_mod.items():
            matching_files = [file for file in files if value in file]
            if matching_files:
                indices_dict[key] = files.index(matching_files[-1]) #-1 index assumes that the last file in matching files is the purity plot
    else:
        indices_dict = {element.replace(f"{sim_name}_", "").replace(".clc", ""): index for index, element in enumerate(files) if sim_name in element}
    
    print("Dictionary of indices:", indices_dict)
    
    if include_LC == False: #remove level control elements
        keys_to_remove = [key for key in indices_dict.keys() if 'LC' in key]
        for key in keys_to_remove:
            del indices_dict[key]
        len_keys_to_remove = len(keys_to_remove)
    
    
    num_keys = len(indices_dict)
    if 'both' in purity_scale:
        num_keys = num_keys + 1
    fig, axs = plt.subplots(num_keys, 1, figsize=(12,4.3 * (num_keys)), sharex=True, dpi=500)
    
    
    # Iterate over the values for each respective key and plot on corresponding subplots
    #       key: subplot name (eg: ".Fd", "CC_xB"...)
    #       value: the index of each data file in the "directory_path" (embedded in clc.extractor)
    
    axs = [axs] if num_keys == 1 else axs
    for ax, (key, value) in zip(axs, indices_dict.items()):
    
        data = export_data(value)
        # Split data['y'] into separate lists
        separated_y = [list(y) for y in zip(*data['y'])]
        print('SUBPLOT: '+key)
        # Plot each separated list with a separate label
        for variable, label in zip(separated_y, data['y_names']):
            print('    '+label)
            if 'LC' in key and include_LC == False: #Exclude LC if False
                continue    
            if '.Pos' in label: #If you manually enter a control variable (valve .pos) into a sheet, switch it with "OP", as it is the standard CV string in this script
                label = 'OP'
    
            #CUT time axis
            time_data = data['x'][:len(variable)] #raw time data
            time_0_index = np.argmin((time_data - time_0*np.ones(len(variable)))**2) #Finding the time value closest to time_0
            if time_end==-1:
                time_end_index  = np.argmin((time_data - time_data[-1]*np.ones(len(variable)))**2)
            elif time_end<0 or time_end<time_0:
                print('ERROR: time_end must be larger than 0 and time_0')
            else: 
                time_end_index  = np.argmin((time_data - time_end*np.ones(len(variable)))**2)
            time = time_data[time_0_index:time_end_index]-time_0*np.ones(len(variable))[time_0_index:time_end_index] #time data for plots
            var = variable[time_0_index:time_end_index] #variable data for plots
            
            #PURITY AXIS scale
            if purity_scale == 'linear':
                xs_y = var
                xs_y_label = 'Mole fraction'
                xs_y_range = [0.9955,1.001]
            elif purity_scale == 'log10':
                xs_y = -np.log10(1*np.ones(len(var))-var)
                xs_y_label = r'Mole fraction'+'\n[$-log_{10}(1-x_{i})$]'
                xs_y_range = [1.9,4.6]
            elif 'both' in purity_scale:
                xs_y = -np.log10(1*np.ones(len(var))-var)
                xs_label = r'Mole fraction'+'\n[$-log_{10}(1-x_{i})$]'
                xs_y_range = [2.1,4.6]
                xs_y2 = var
                xs_label2 = 'Mole fraction'
                xs_y_range = [0.994,1.001]
            
            
            #Rename SUBPLOT LABELS to reasonable names
            if 'PV' in label: #process variable
                plot_label = 'Process variable'
            elif 'SP' in label: #setpoint
                plot_label = 'Setpoint'
            elif 'OP' in label: #ax2, controller output OP
                plot_label = 'Controller output'
            elif 'Zn("WATER")' in label:
                plot_label = r'$x_{B,water}$'
            elif 'Zn("METHANOL")' in label:
                plot_label = r'$x_{D,methanol}$'
            else:
                plot_label = 'unknown'
            
            title_key = ''
            y2_label = ''
            y_label = ''
            
            #Rename SUBPLOT TITLES and SUBPLOT AXIS to reasonable names
            if "Fd" in key: #disturbance
                y_label = 'Feed flow (F) [kmol/h]\n (disturbance)'
            elif "FC_L" in key: #reflux flow
                if 'LF' in key:
                    title_key ='Reflux flow (L) (Ratio L/F)'
                else:
                    title_key = 'Reflux flow (L)'
                y2_label = '\n [kg/h]'
                y_label = '[kg/h]'
            elif "FC_V" in key: #reboiler duty
                if 'VF' in key:
                    title_key = 'Reboil vapour flow (V) (Ratio V/F)'
                    y2_label = '\n[kW]'
                else:
                    title_key = 'Reboil vapour flow (V)'
                    y2_label = '\n[kg/h]'
                y_label = ' [kmol/h]'
            elif 'CC' in key: #composition control
                if 'xB' in key:
                    title_key = r'$x_{B}$ control'
                    y2_label = '\nreboil duty [kW]'#Control variable xs ax2
                if 'xD' in key:
                    title_key = r'$x_{D}$ control'
                    y2_label = '\nreflux flow [kg/h]'#Control variable xs ax2
                y_label = xs_y_label
            elif "xs" in key or "AD" in key and "Zn" in label: #purities
                if only_xs:
                    title_key = ''
                    y_label = f'${key}$'
                elif 'both' in purity_scale:
                    title_key2 = ''
                    y_label2 = f'${key}$'
                else:
                    title_key = r'Purities ($x_i$)'
                    y_label = xs_y_label
            elif "LC" in key: #level control
                if '_D' in key:
                    title_key = 'Reflux drum level'
                    y2_label = f'\nDistillate [valve\%]'
                if '_B' in key:
                    title_key = 'Sump (reboiler) level'
                    y2_label = f'\nBottoms [valve\%]'
                y_label = 'Level [m]'
            elif "PC" in key: #pressure control
                if '_D' in key:
                    title_key = 'Reflux drum pressure'
                    y2_label = f'\nCoolant flow [kg/hr]'
                if '_B' in key:
                    title_key = 'Sump (reboiler) level'
                    y2_label = f'\nBottoms [valve\%]'
                y_label = 'Pressure [bar]' 
            else:
                title_key = 'title key error'
                y2_label = 'ax2 label error'
                y_label = 'ax label error'
                
            
            #Plot logic: what and where to plot
            if 'OP' in label and 'Fd' not in key or 'QRebR' in label:
                ax2 = ax.twinx() # Create a secondary y-axis for 'OP' variable
                ln2 = ax2.plot(time, var, label=plot_label , color='green',linestyle='-.',linewidth = 3 ,alpha=0.5)
                ax2.set_ylabel(plot_label +' '+ y2_label, color='green')
                ax2.tick_params('y', colors='green')
            elif 'CC' in key:
                if 'SP' in label:
                    ln1 = ax.plot(time, xs_y, label=plot_label, color='grey',linestyle='-.')
                    ax.set_ylabel(y_label)
                else:
                    ax.plot(time, xs_y, label=plot_label)    
                    ax.set_ylabel(y_label)
            elif 'Fd' in key: #only include the process variable for the feed disturbance
                if 'PV' in label:
                    ln1 = ax.plot(time, var, label=plot_label,linestyle='-.')
                    ax.set_ylabel( y_label)
                else:
                    None
            elif 'xs' in key or 'Zn' in label:                
                xs_y = np.array(xs_y)
                xs_filtered = np.where((xs_y >= xs_y_range[0]) & (xs_y <= xs_y_range[1]),xs_y,None)
                ln1 = ax.plot(time, xs_filtered, label=plot_label) #-LOG SCALE: -np.log10(1*np.ones(len(variable))-variable)
                ax.set_ylim(xs_y_range[0],xs_y_range[1])
                if only_xs:
                    ax.set_ylabel(f'${key}$')
                else:
                    ax.set_ylabel(y_label)
                if 'both' in purity_scale:
                    xs_y = np.array(xs_y)
                    xs_filtered = np.where((xs_y >= xs_y_range[0]) & (xs_y <= xs_y_range[1]),xs_y,None)
                    ln1 = ax.plot(time, xs_filtered, label=plot_label) #-LOG SCALE: -np.log10(1*np.ones(len(variable))-variable)
                    ax.set_ylim(xs_y_range[0],xs_y_range[1])
                    ax.set_ylabel(y_label)
            else:
                if 'SP' in label:
                    ax.plot(time, var, label=plot_label, color='grey',linestyle='-.')
                    ax.set_ylabel(y_label)
                else:
                    ln1 = ax.plot(time, var, label=plot_label)
                ax.set_ylabel(y_label)
            try:
                lns = ln1 + ln2
                labs = [l.get_label() for l in lns]
                ax.legend(lns, labs, loc='best')
            except NameError:
                lns = ln1
                labs = [l.get_label() for l in lns]
                ax.legend(lns, labs, loc='best')
                
            # axis number format
            if tick_format == '':
                pass
            else:
                ax.yaxis.set_major_formatter(FuncFormatter(format_ticks))
                ax.xaxis.set_major_formatter(FuncFormatter(format_ticks))
            
        if 'xs' in key or 'Zn' in label:
            if purity_scale == 'log10':
                xs_hline = -np.log10(1-purity)
            elif purity_scale == 'linear':
                xs_hline = purity            
            elif 'both' in purity_scale:
                xs_hline = -np.log10(1-purity)
                xs_hline2 = purity 
                axs[-1].axhline(y=xs_hline2, color='r', linestyle='-.', alpha=0.5, label = f'Target purity ({purity*100:.2f} mole\%)')
            
            ax.axhline(y=xs_hline, color='r', linestyle='-.', alpha=0.5, label = f'Target purity ({purity*100:.2f} mole\%)')
            
        ax.set_xlim(left=0,right=max(data['x'][:len(var)]))
        ax.grid(True, linestyle='--', alpha=0.9)
        ax.set_title(f'{title_key}')
        ax.legend(loc='best', fancybox=True, framealpha=0.5, handlelength=1.5, handleheight=1.6, ncol=1, labelspacing=0.0, columnspacing=0, handletextpad=1, edgecolor='black', facecolor='white', markerscale=2)
    
    # Set common xlabel for all subplots
    axs[-1].set_xlabel(f"Time ({time_unit})")
    
    #sim name specific TITLES
    if 'test' in sim_name:
        title = 'Feed disturbance on $x_D$ and $x_B$'
    elif only_xs:
        title = "All purities" + f' ({purity_scale}, mole fraction)'
    elif "AD0" in sim_name :
        title = "AD0: No control"
    elif "AD1" in sim_name :
        title = "AD1: Ratio control L/F with V constant"
    elif "AD2" in sim_name :
        title = "AD2: Ratio control L/F and V/F"
    elif "AD3" in sim_name :
        title = "AD3: One compositon loop.  L $\leftrightarrow x_{D}$"
    elif "AD4" in sim_name :
        title = "AD4: Two compositon loops. \nL $\leftrightarrow x_{D}$,  V $\leftrightarrow x_{B}$"
    elif "AD5" in sim_name :
        title = "AD5: Combined ratio and feedback"
    elif "AD7" in sim_name :
        title = "AD6: Ratio and composition cascade"
    else:
        title = sim_name
            
            
            
        
    axs[0].set_title(f"{title}",fontsize=40)
    
    # plt.tight_layout()
    plt.show()
    return

objects = list(xs_dict.values())
plot(sim_name)
# for obj in objects:
#     plot(obj)
